WRITING CONTROL STRUCTURES
----------------------------------

/*
1) IF STATEMENTS
-----------------

SYNTAX:

IF <condition> THEN
  statement1;
  statement2;
  ...

[ELSIF <condition> THEN
  statement1;
  statement2;
  ...]

[ELSE 
  statement1;
  statement2;
  ...]

END IF;

*/

-- EXAMPLE 1


DECLARE 
  x INT;
BEGIN
  x:= 20;
  
  IF x = 10 THEN
  
    DBMS_OUTPUT.PUT_LINE('X IS EQUAL TO 10');    
    
  ELSE

    DBMS_OUTPUT.PUT_LINE('X IS NOT EQUAL TO 10');    
  
  END IF;

END;

-- EXAMPLE 2

DECLARE
  v_date DATE := '01-JAN-2009';
  v_diff BOOLEAN;
  v_name CHAR(10);
BEGIN
  
  IF MONTHS_BETWEEN(SYSDATE,v_date) /12 > 5 THEN
    v_diff := TRUE;
    v_name := 'YES';
  ELSE
    v_diff := FALSE;
    v_name := 'NO';
  END IF;
  
  DBMS_OUTPUT.PUT_LINE('MORE THAN 5 YEARS? ' || TO_CHAR(v_name) );    

END;


-- EXAMPLE 3


select * from HR.EMPLOYEES WHERE EMPLOYEE_ID = 176;

DECLARE
  v_dpto HR.EMPLOYEES.DEPARTMENT_ID%TYPE;
  v_bonus HR.EMPLOYEES.SALARY%TYPE;

BEGIN
  SELECT DEPARTMENT_ID, SALARY
  INTO v_dpto, v_bonus
  FROM HR.EMPLOYEES
  WHERE EMPLOYEE_ID = 176;
  
  IF v_dpto = '10' THEN
    v_bonus:= v_bonus  + 10;
  
  ELSIF v_dpto = '80'  THEN
    v_bonus:= v_bonus  + 20;
  
  ELSE
    v_bonus:= v_bonus  + 30;
  
  END IF;
  DBMS_OUTPUT.PUT_LINE('DEPARTMENT: ' || TO_CHAR(v_dpto) ||  ' NEW SALARY: ' || TO_CHAR(v_bonus));    
END;


-- EXAMPLE 4
select LAST_NAME
     , SALARY
     , department_ID
from HR.EMPLOYEES 
WHERE department_id = 10;

DECLARE
  v_name  HR.EMPLOYEES.LAST_NAME%TYPE;
  v_bonus HR.EMPLOYEES.salary%TYPE;

BEGIN
  SELECT LAST_NAME, salary
  INTO v_name, v_bonus
  FROM HR.EMPLOYEES
  WHERE department_id = 10;
  
  IF v_bonus > 100 AND SUBSTR(v_name,1,1) = 'W' THEN
    v_bonus:= v_bonus  + 10;
  
  ELSE
    v_bonus:= v_bonus  + 30;
  
  END IF;
  
  DBMS_OUTPUT.PUT_LINE('NEW SALARY: ' || TO_CHAR(v_bonus));    

END;




/*
2) CASE EXPRESSIONS
-------------------  

*/

SYNTAX:

CASE <selector>
  WHEN <expression1> THEN <statement1>
  WHEN <expression2> THEN <statement2>
  WHEN <expression3> THEN <statement3>
  ...
  [ELSE statementN]
END;

-- USE ONLY THE = OPERATOR
-- TAKE SPECIAL CARE WITH NULLS!

-- EXAMPLE 1

DECLARE
  v_grade CHAR(1) := 'C';
  v_comment VARCHAR2(20);
BEGIN
  v_comment :=
    CASE v_grade
      WHEN 'A' THEN 'Excelent'
      WHEN 'B' THEN 'Very Good'
      WHEN 'C' THEN 'Good'
      ELSE 'There is a problem'
    END;
    
    DBMS_OUTPUT.PUT_LINE('Grade: ' || TO_CHAR(v_grade) || ' Comment: ' || TO_CHAR(v_comment) );    

END;


/*
3) LOOP STATEMENTS
------------------

*/

-- BASIC LOOP
-- SYNTAX:
LOOP
  <statement1>;
  <statement2>;
  <statement3>;
  ...
  
  EXIT [WHEN <condition>];

END LOOP;

-- AT LEAST ONE EXECUTION OF THE STATEMENTS

-- EXAMPLE 1:

DECLARE
  x INT;
BEGIN

  x := 0;
  
  LOOP
    
    DBMS_OUTPUT.PUT_LINE('VALUE OF X: ' || TO_CHAR(X));    
    
    X := X +1;
    
    EXIT WHEN X > 10;
    
  END LOOP;

END;

 

-- WHILE LOOP
-- SYNTAX:
WHILE <condition> LOOP
  <statement1>;
  <statement2>;
  <statement3>;
  ...

END LOOP;

-- 0, 1 OR MORE EXECUTIONS

-- EXAMPLE:

  SELECT *
  FROM HR.LOCATIONS
  WHERE country_id = 'CA'


DECLARE
  v_country_id  HR.LOCATIONS.country_id%TYPE := 'CA';
  v_location_id HR.LOCATIONS.location_id%TYPE;
  v_count     NUMBER:=1;
BEGIN

  SELECT MAX(location_id)
  INTO v_location_id
  FROM HR.LOCATIONS
  WHERE country_id = v_country_id;
  
  WHILE v_count <= 3 LOOP
    DBMS_OUTPUT.PUT_LINE('City');    
  
    v_count := v_count + 1;
  END LOOP;

END;


-- FOR LOOP
-- SYNTAX:

FOR counter IN [REVERSE] lower...upper LOOP
  <statement1>;
  <statement2>;
  <statement3>;
  ...
END LOOP;

-- DO NOT DECLARE THE COUNTER
-- COUNTER WITHIN THE LOOP ONLY

-- EXAMPLE 1

BEGIN
  
  FOR counter IN 1..5 LOOP
  
    DBMS_OUTPUT.PUT_LINE('Counter: ' || TO_CHAR(counter));    
    
  END LOOP; 

END;


-- EXAMPLE 2

DECLARE
  lower NUMBER := 1;
  upper NUMBER := 100;
BEGIN
  
  FOR i IN lower..upper LOOP
  
    DBMS_OUTPUT.PUT_LINE('Counter: ' || TO_CHAR(i));    
    
  END LOOP; 

END;



-- NESTED LOOPS AND LABELS

-- USE NESTED LOOPS WITH CAUTION
-- ASSIGN LABELS TO LOOPS
-- EXIT SPECIFIC LOOP OR ALL NESTED LOOPS

-- EXAMPLE 1

BEGIN
  
  <<loop1>>
  FOR counter IN 1..5 LOOP
  
    DBMS_OUTPUT.PUT_LINE('Counter: ' || TO_CHAR(counter));    
    
  END LOOP loop1; 

END;

-- EXAMPLE 2

BEGIN
  
  <<loop1>>
  FOR i IN 1..5 LOOP
  
    <<loop2>>
    FOR j IN 1..10 LOOP
    
      DBMS_OUTPUT.PUT_LINE('ixj: ' || TO_CHAR(i*j));    
    END LOOP loop2; 
    
  END LOOP loop1; 

END;
